@@ -22,6 +22,7 @@ from typing import Union, List, Dict
 import pandas as pd
 import os
 import time
+import random
 import json
 
 
@@ -572,6 +573,7 @@ class SagemakerFeatureStore:
         feature_group_name: str,
         paritioned_spark_df: SparkDataFrame,
         thread_num: int = 10,
+        redis_expiration: int = 604800,
     ):
         """
         Ingests a partitioned Spark DataFrame into the Feature Group.
@@ -600,6 +602,7 @@ class SagemakerFeatureStore:
             redis_config,
             ingest_to_aws,
             thread_num=10,
+            redis_expiration=604800,
         ):
             if redis_config:
                 redis_client = Redis(
@@ -648,15 +651,17 @@ class SagemakerFeatureStore:
                             for column in row.__fields__
                         }
                         pipe.hmset(key_name, mapping)
-                        # expire for 7 days
-                        pipe.expire(key_name, 604800)
+                        # expire for 7 days (default)
+                        pipe.expire(key_name, redis_expiration)
 
                         ops = ops + 1
                         if ops == ops_limit:
+                            time.sleep(1 * (0.8 + random.random() * 0.4))
                             pipe.execute()
                             pipe = redis_client.pipeline()
                             ops = 0
                 if is_redis and ops > 0:
+                    time.sleep(1 * (0.8 + random.random() * 0.4))
                     pipe.execute()
                 print("Done writing to Redis")
 
@@ -710,12 +715,15 @@ class SagemakerFeatureStore:
                     redis_config,
                     ingest_to_aws,
                     thread_num,
+                    redis_expiration,
                 )
             )
         except Exception as e:
             raise Exception("ERROR:", e)
 
-    def ingest_batch_online(self, feature_group_name: str, data: List[dict]):
+    def ingest_batch_online(
+        self, feature_group_name: str, data: List[dict], redis_expiration: int = 604800
+    ):
         """
         Ingests a list of dictionaries to the feature group's online store. Limited to <=100 records per call.
         Example: `[{"feat_1": "value", "feat_2": "value"}, {"feat_1": "value", "feat_2": "value"}]`
@@ -762,6 +770,8 @@ class SagemakerFeatureStore:
 
         if is_redis:
             pipe = self.redis_client.pipeline()
+
+        ops_ctr = 0
         for record in data:
             # Ingest to AWS Dynamodb sagemaker
             if is_dynamodb:
@@ -791,9 +801,17 @@ class SagemakerFeatureStore:
                     elif value is None:
                         record[key] = ""
                 pipe.hmset(key_name, record)
-                # Expire for 7 days
-                pipe.expire(key_name, 604800)
-        if is_redis:
+                # expire for 7 days (default)
+                pipe.expire(key_name, redis_expiration)
+                ops_ctr = ops_ctr + 1
+                if ops_ctr == 3200:
+                    time.sleep(1 * (0.8 + random.random() * 0.4))
+                    pipe.execute()
+                    # Reset
+                    pipe = self.redis_client.pipeline()
+                    ops_ctr = 0
+        if is_redis and ops_ctr > 0:
+            time.sleep(1 * (0.8 + random.random() * 0.4))
             pipe.execute()
             print("Done writing to redis")
 
@@ -1066,6 +1084,7 @@ class SagemakerFeatureStore:
         feature_group_name: str,
         record_identifier_name: str,
         ops_limit: int = 3200,
+        redis_expiration: int = 604800,
     ):
         """
         Initializes a Redis connection
@@ -1106,15 +1125,18 @@ class SagemakerFeatureStore:
                 elif value is None:
                     row[key] = ""
             pipe.hmset(key_name, row)
-            pipe.expire(key_name, 604800)
+            # expire for 7 days (default)
+            pipe.expire(key_name, redis_expiration)
 
             ops = ops + 1
             if ops == ops_limit:
+                time.sleep(1 * (0.8 + random.random() * 0.4))
                 pipe.execute()
                 # Reset
                 pipe = self.redis_client.pipeline()
                 ops = 0
         if ops > 0:
+            time.sleep(1 * (0.8 + random.random() * 0.4))
             pipe.execute()
 
         print("Done writing to Redis!")
